import cp from 'node:child_process';
import fs from 'node:fs';
import path from 'node:path';
import process from 'node:process';
import { sync as glob } from 'fast-glob';
import normalize from 'normalize-path';
export const invokeUpdate = (cwd, args) => {
    var _a;
    const isWin = ['win32' || 'win64'].includes(process.platform);
    const stdio = ['inherit', 'inherit', 'inherit'];
    const cmd = isWin ? 'npx.cmd' : 'npx';
    const result = cp.spawnSync(cmd, ['npm-upgrade', ...args], { cwd: normalize(cwd), stdio });
    if (result.error || result.status || result.signal !== null) {
        throw result;
    }
    return (_a = result.stdout) === null || _a === void 0 ? void 0 : _a.toString().trim();
};
export const resolveWorkspaces = (cwd, patterns) => {
    const entries = glob(patterns, {
        cwd,
        onlyDirectories: true,
        absolute: true,
    });
    return Object.keys(entries.reduce((m, entry) => {
        try {
            const stat = fs.lstatSync(entry);
            if (stat.isFile()) {
                m[path.dirname(entry)] = true;
            }
            if (stat.isDirectory()) {
                m[entry] = true;
            }
        }
        catch (_a) { }
        return m;
    }, {}));
};
export const readPkg = (cwd) => JSON.parse(fs.readFileSync(path.resolve(cwd, 'package.json'), { encoding: 'utf8' }));
export const extractWorkspacesFromPkg = (pkg) => pkg.workspaces
    ? Array.isArray(pkg.workspaces)
        ? pkg.workspaces
        : pkg.workspaces.packages
    : [];
export const getWorkspaces = (cwd, ws) => {
    const pattern = typeof ws === 'string'
        ? ws.split(',')
        : Array.isArray(ws)
            ? ws
            : extractWorkspacesFromPkg(readPkg(cwd));
    return resolveWorkspaces(cwd, pattern);
};
export const upgrade = (cwd, flags) => {
    const workspaces = [cwd, ...getWorkspaces(cwd, flags.w || flags.workspaces)];
    workspaces.forEach((ws) => {
        if (fs.existsSync(path.resolve(ws, 'package.json'))) {
            console.log(`invoke npm-upgrade for ${ws}`);
            invokeUpdate(ws, process.argv.slice(2));
        }
        else {
            console.warn(`${ws} package.json not found`);
        }
    });
};
